*****************************************************************
*                                                               *
*   PACKAGE OF ROUTINES FOR EVALUATING SPHERICAL HARMONICS      *
*                                                               *
*            Y(L,M)=BETT(L,M)*PLJ(L,M)*EXP(I*M*FI)              *
*                                                               *
*****************************************************************
*                                                               *
*                        / COS(M*FI) ,M>0                       *
*      YY(L,M)=PLJ(L,N)*<                  ; N=ABS(M)           *
*                        \ SIN(N*FI) ,M<0                       *
*                                                               *
*      PLJ(L,N)-AUGEMENTED LEGANDRE POLINOMS                    *
*                                                               *
*     PARAMETERS :                                              *
*       X,Y,Z - COMPONENTS OF THE UNIT VECTOR                   *
*       ILS - MAXIMUM VALUE OF THE ORBIT NUMBER                 *
*       YLM - RESULTING ARRAY                                   *
*                                                               *
*  Obtained from K.Hummler in Max-Planck-Institut, Germany      *
*****************************************************************
	SUBROUTINE SPHHARM(X0,Y0,Z0,ILS,YLM)
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	PARAMETER (LPLMAX=(2*LMAXV+3)**2)
	COMPLEX*16 YLM(LPLMAX)
	DIMENSION YY(LPLMAX),BETT(LPLMAX)
	DIMENSION C(2*LMAXV+3),S(2*LMAXV+3),P(2*LMAXV+3,2*LMAXV+3)
	EQUIVALENCE (X,C(2)),(Y,S(2)),(Z,P(2,1))
	SAVE IF,BETT,C,S,P
	DATA C(1),S(1),P(1,1),P(2,2) /1.D0,0.D0,1.D0,1.D0/,IF/0/
	IF(IF.EQ.0)CALL CNSBET(2*LMAXV+2,BETT)
	IF=1
      N = (ILS+1)**2
      YY(1) = 1.D0
      X = X0
      Y = Y0
      Z = Z0
  500 ST = X*X + Y*Y
      Z2 = Z*Z
      R2 = ST+Z2
      IF (N .LT. 2) GOTO 99
      YY(2) = Y
      YY(3) = Z
      YY(4) = X
      NT = 1
      DO  5  L = 2, ILS
	LP1 = L+1
	LM1 = L-1
	LAV = L*LP1 + 1
	P(LP1,1) = ((L+LM1)*Z*P(L,1) - LM1*R2*P(LM1,1)) / L
	YY(LAV) = P(LP1,1)
	NT = NT+2
	P(LP1,LP1) = P(L,L)*NT
	C(LP1) = X*C(L) - Y*S(L)
	S(LP1) = X*S(L) + Y*C(L)
	LAVPL = LAV+L
	YY(LAVPL) = P(LP1,LP1)*C(LP1)
	LAVML = LAV-L
	YY(LAVML) = P(LP1,LP1)*S(LP1)
	IF (ST .GT. Z2) GOTO 3
	DO  2  LMM = 1, LM1
	  M = L-LMM
	  LAVPM = LAV+M
	  LAVMM = LAV-M
	  MP1 = M+1
	  P(LP1,MP1) = (R2*(L+M)*P(L,MP1) - ST*P(LP1,MP1+1)) / (Z*(L-M))
	  YY(LAVPM) = P(LP1,MP1)*C(MP1)
	  YY(LAVMM) = P(LP1,MP1)*S(MP1)
    2   CONTINUE
	GOTO 5
    3   DO  4  M = 1, LM1
	  MP1 = M+1
	  LAVPM = LAV+M
	  LAVMM = LAV-M
	  P(LP1,MP1) = ((LM1+M)*R2*P(L,M) - (LP1-M)*Z*P(LP1,M))/ST
	  YY(LAVPM) = P(LP1,MP1)*C(MP1)
	  YY(LAVMM) = P(LP1,MP1)*S(MP1)
    4   CONTINUE
    5 CONTINUE
C  Inserted by S.S -->
 99   CONTINUE
	DO 20 L=0,ILS
	L0=L*(L+1)+1
	YLM(L0)=YY(L0)*BETT(L0)
	DO 10 M=1,L
	LM=L0+M
	LMM=L0-M
	YLM(LM)=DCMPLX(YY(LM),YY(LMM))*BETT(LM)
	YLM(LMM)=DCMPLX(YY(LM),-YY(LMM))*BETT(LMM)
  10    CONTINUE
  20    CONTINUE
C
	RETURN
	END



	SUBROUTINE REHARM(X0,Y0,Z0,ILS,YY)
*****************************************************************
*  Calculates unnormalized real spherical harmonics.            *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	PARAMETER (LPLMAX=(2*LMAXV+3)**2)
	DIMENSION YY(LPLMAX)
	DIMENSION C(2*LMAXV+3),S(2*LMAXV+3),P(2*LMAXV+3,2*LMAXV+3)
	EQUIVALENCE (X,C(2)),(Y,S(2)),(Z,P(2,1))
	SAVE C,S,P
	DATA C(1),S(1),P(1,1),P(2,2) /1.D0,0.D0,1.D0,1.D0/
      N = (ILS+1)**2
      YY(1) = 1.D0
      X = X0
      Y = Y0
      Z = Z0
  500 ST = X*X + Y*Y
      Z2 = Z*Z
      R2 = ST+Z2
      IF (N .LT. 2)RETURN 
      YY(2) = Y
      YY(3) = Z
      YY(4) = X
      NT = 1
      DO  5  L = 2, ILS
	LP1 = L+1
	LM1 = L-1
	LAV = L*LP1 + 1
	P(LP1,1) = ((L+LM1)*Z*P(L,1) - LM1*R2*P(LM1,1)) / L
	YY(LAV) = P(LP1,1)
	NT = NT+2
	P(LP1,LP1) = P(L,L)*NT
	C(LP1) = X*C(L) - Y*S(L)
	S(LP1) = X*S(L) + Y*C(L)
	LAVPL = LAV+L
	YY(LAVPL) = P(LP1,LP1)*C(LP1)
	LAVML = LAV-L
	YY(LAVML) = P(LP1,LP1)*S(LP1)
	IF (ST .GT. Z2) GOTO 3
	DO  2  LMM = 1, LM1
	  M = L-LMM
	  LAVPM = LAV+M
	  LAVMM = LAV-M
	  MP1 = M+1
	  P(LP1,MP1) = (R2*(L+M)*P(L,MP1) - ST*P(LP1,MP1+1)) / (Z*(L-M))
	  YY(LAVPM) = P(LP1,MP1)*C(MP1)
	  YY(LAVMM) = P(LP1,MP1)*S(MP1)
    2   CONTINUE
	GOTO 5
    3   DO  4  M = 1, LM1
	  MP1 = M+1
	  LAVPM = LAV+M
	  LAVMM = LAV-M
	  P(LP1,MP1) = ((LM1+M)*R2*P(L,M) - (LP1-M)*Z*P(LP1,M))/ST
	  YY(LAVPM) = P(LP1,MP1)*C(MP1)
	  YY(LAVMM) = P(LP1,MP1)*S(MP1)
    4   CONTINUE
    5 CONTINUE
	END


	SUBROUTINE CNSBET(ILS,BETT)
***************************************************************
*                                                             *
*   PROGRAM FOR CALCULATING THE NORM. COEFFICIENTS BETT       *
*                                                             *
*               / (2L+1)*(L-N)! \ (1/2)                       *
*    BETT(L,M)=(  -------------  )      ; N=ABS(M)            *
*               \  4*PI *(L+N)! /                             *
*                                                             *
*  Written by S.Yu.Savrasov (P.N.Lebedev Physical Institute)  *
***************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION BETT((ILS+1)*(ILS+1))
	DATA PI/3.141592653589793D0/
	DO 20 L=0,ILS
	DO 10 M=-L,L
	LM=L*(L+1)+M+1
	MSIGN=(-1)**((M+ABS(M))/2)   !! PHASE AFTER CONDON AND SHORTLEY
	F=FACT12(L-ABS(M),L+ABS(M))
	BETT(LM)=SQRT((2.D0*L+1.D0)/4.D0/PI*F)*MSIGN
  10    CONTINUE
  20    CONTINUE
	END


C***************************************************
	FUNCTION FACT12(N,M)   !!!   N!/M!
C***************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	IF(N.LT.M)THEN
	FACT12=1.D0/DBLE(N+1)
	DO 10 I=N+2,M
  10    FACT12=FACT12/DBLE(I)
	ELSEIF(N.EQ.M)THEN
	FACT12=1.D0
	ELSEIF(N.GT.M)THEN
	FACT12=DBLE(M+1)
	DO 20 I=M+2,N
  20    FACT12=FACT12*DBLE(I)
	ENDIF
	END
